@@ -24,7 +24,8 @@ use fs::Fs;
 use futures::{future::join_all, StreamExt};
 use gpui::{
     list, AnyElement, AppContext, AsyncWindowContext, ClickEvent, EventEmitter, FocusHandle,
-    FocusableView, ListAlignment, ListState, Model, Render, Task, View, WeakView,
+    FocusableView, ListAlignment, ListState, Model, ReadGlobal, Render, Task, UpdateGlobal, View,
+    WeakView,
 };
 use language::{language_settings::SoftWrap, LanguageRegistry};
 use markdown::{Markdown, MarkdownStyle};
@@ -124,7 +125,7 @@ impl AssistantPanel {
             })?;
 
             cx.new_view(|cx| {
-                let project_index = cx.update_global(|semantic_index: &mut SemanticIndex, cx| {
+                let project_index = SemanticIndex::update_global(cx, |semantic_index, cx| {
                     semantic_index.project_index(project.clone(), cx)
                 });
 
@@ -288,7 +289,7 @@ impl AssistantChat {
         workspace: WeakView<Workspace>,
         cx: &mut ViewContext<Self>,
     ) -> Self {
-        let model = CompletionProvider::get(cx).default_model();
+        let model = CompletionProvider::global(cx).default_model();
         let view = cx.view().downgrade();
         let list_state = ListState::new(
             0,
@@ -439,7 +440,7 @@ impl AssistantChat {
                     Markdown::new(
                         text,
                         self.markdown_style.clone(),
-                        self.language_registry.clone(),
+                        Some(self.language_registry.clone()),
                         cx,
                     )
                 });
@@ -550,7 +551,7 @@ impl AssistantChat {
                 let messages = messages.await?;
 
                 let completion = cx.update(|cx| {
-                    CompletionProvider::get(cx).complete(
+                    CompletionProvider::global(cx).complete(
                         model_name,
                         messages,
                         Vec::new(),
@@ -572,7 +573,7 @@ impl AssistantChat {
                                         Markdown::new(
                                             "".into(),
                                             this.markdown_style.clone(),
-                                            this.language_registry.clone(),
+                                            Some(this.language_registry.clone()),
                                             cx,
                                         )
                                     }),
@@ -666,7 +667,7 @@ impl AssistantChat {
                     Markdown::new(
                         "".into(),
                         self.markdown_style.clone(),
-                        self.language_registry.clone(),
+                        Some(self.language_registry.clone()),
                         cx,
                     )
                 }),
@@ -682,7 +683,7 @@ impl AssistantChat {
                     Markdown::new(
                         "".into(),
                         self.markdown_style.clone(),
-                        self.language_registry.clone(),
+                        Some(self.language_registry.clone()),
                         cx,
                     )
                 }),
