@@ -123,6 +123,7 @@ impl Channel {
     }
 }
 
+#[derive(Debug)]
 pub struct ChannelMembership {
     pub user: Arc<User>,
     pub kind: proto::channel_member::Kind,
@@ -815,9 +816,11 @@ impl ChannelStore {
             Ok(())
         })
     }
-    pub fn get_channel_member_details(
+    pub fn fuzzy_search_members(
         &self,
         channel_id: ChannelId,
+        query: String,
+        limit: u16,
         cx: &mut ModelContext<Self>,
     ) -> Task<Result<Vec<ChannelMembership>>> {
         let client = self.client.clone();
@@ -826,26 +829,24 @@ impl ChannelStore {
             let response = client
                 .request(proto::GetChannelMembers {
                     channel_id: channel_id.0,
+                    query,
+                    limit: limit as u64,
                 })
                 .await?;
-
-            let user_ids = response.members.iter().map(|m| m.user_id).collect();
-            let user_store = user_store
-                .upgrade()
-                .ok_or_else(|| anyhow!("user store dropped"))?;
-            let users = user_store
-                .update(&mut cx, |user_store, cx| user_store.get_users(user_ids, cx))?
-                .await?;
-
-            Ok(users
-                .into_iter()
-                .zip(response.members)
-                .map(|(user, member)| ChannelMembership {
-                    user,
-                    role: member.role(),
-                    kind: member.kind(),
-                })
-                .collect())
+            user_store.update(&mut cx, |user_store, _| {
+                user_store.insert(response.users);
+                response
+                    .members
+                    .into_iter()
+                    .filter_map(|member| {
+                        Some(ChannelMembership {
+                            user: user_store.get_cached_user(member.user_id)?,
+                            role: member.role(),
+                            kind: member.kind(),
+                        })
+                    })
+                    .collect()
+            })
         })
     }
 
