@@ -509,8 +509,7 @@ pub type NotificationBatch = Vec<(UserId, proto::Notification)>;
 
 pub struct CreatedChannelMessage {
     pub message_id: MessageId,
-    pub participant_connection_ids: Vec<ConnectionId>,
-    pub channel_members: Vec<UserId>,
+    pub participant_connection_ids: HashSet<ConnectionId>,
     pub notifications: NotificationBatch,
 }
 
