@@ -13,6 +13,7 @@ mod list;
 mod modal;
 mod popover;
 mod popover_menu;
+mod radio;
 mod right_click_menu;
 mod stack;
 mod tab;
@@ -39,6 +40,7 @@ pub use list::*;
 pub use modal::*;
 pub use popover::*;
 pub use popover_menu::*;
+pub use radio::*;
 pub use right_click_menu::*;
 pub use stack::*;
 pub use tab::*;
